/***********************************************************************

  File: dac.v

  This is a synthesizable delta-sigma DAC, discussed in the Xilinx 
  XAPP154 application note.  A low pass filter on the output will
  improve the audio quality, but for this lab it is not required.

***********************************************************************/

`timescale 1 ns / 1 ps

module dac (
                sample,
                analog,
                clk,
                rst
                );

  input   [7:0] sample;
  output        analog;
  input         clk;
  input         rst;


  //******************************************************************//
  // Implement the delta-sigma converter.                             //
  //******************************************************************//


  wire [7:0] UnsignedSample;
  wire [9:0] DeltaAdder;
  wire [9:0] SigmaAdder;
  wire [9:0] DeltaB;
  reg  [9:0] Sigma;

  assign UnsignedSample = {!sample[7], sample[6:0]};
  assign DeltaB = {Sigma[9], Sigma[9]} << 8;
  assign SigmaAdder = DeltaAdder + Sigma;
  assign DeltaAdder = UnsignedSample + DeltaB;

  always @(posedge clk or posedge rst)
  begin
    if (rst) Sigma <= 1'b1 << 8;
    else Sigma <= SigmaAdder;
  end

  assign analog = Sigma[9];


  //******************************************************************//
  //                                                                  //
  //******************************************************************//


endmodule
